
import std.stdio : writeln;
import ecap, msgq, esrc, edsm, disp;

class Main : StageMachine {
    enum ulong M0_IDLE = 0;
    Timer tm0;

    this() {
        super("MAIN");

        Stage init, idle;
        init = addStage("INIT", &mainInitEnter);
        idle = addStage("IDLE", &mainIdleEnter);
        init.addReflex("M0", idle);
        idle.addReflex("T0", &mainIdleT0);
    }

    void mainInitEnter() {
        tm0 = newTimer();
        msgTo(this, M0_IDLE);
    }

    void mainIdleEnter() {
        tm0.enable();
        tm0.heartBeat(10); // milliseconds
    }

    void mainIdleT0(StageMachine src, Object o) {
        int[] a;
        foreach (k; 0 .. 1000) {
            a ~= k;
        }
    }
}

class Stopper : StageMachine {

    enum ulong M0_IDLE = 0;
    Signal sg0, sg1;

    this() {
        super("STOPPER");

        Stage init, idle;
        init = addStage("INIT", &stopperInitEnter);
        idle = addStage("IDLE", &stopperIdleEnter);
        init.addReflex("M0", idle);
        idle.addReflex("S0", &stopperIdleS0);
        idle.addReflex("S1", &stopperIdleS1);
    }

    void stopperInitEnter() {
        sg0 = newSignal(Signal.sigInt);
        sg1 = newSignal(Signal.sigTerm);
        msgTo(this, M0_IDLE);
    }

    void stopperIdleEnter() {
        sg0.enable();
        sg1.enable();
    }

    void stopperIdleS0(StageMachine src, Object o) {
        stopEventQueue();
    }

    void stopperIdleS1(StageMachine src, Object o) {
        stopEventQueue();
    }
}

void main(string[] args) {

    auto Main = new Main();
    Main.run();

    auto stopper = new Stopper();
    stopper.run();

    writeln(" === Hello, world! === ");
    auto md = new MessageDispatcher();
    md.loop();
    writeln(" === Goodbye, world! === ");
}
